@@ -25,7 +25,7 @@ class FlatpakRemoteInfo():
         self.noenumerate = remote.get_noenumerate()
 
         if not self.title or self.title == "":
-            self.title = name.capitalize()
+            self.title = self.name.capitalize()
 
 _fp_sys = None
 
@@ -53,6 +53,20 @@ def make_pkg_hash(ref):
     except Exception:
         return "fp:%s:%s" % (ref.get_remote_name(), ref.format_ref())
 
+def _get_remote_name_by_url(fp_sys, url):
+    name = None
+
+    try:
+        remotes = fp_sys.list_remotes()
+    except GLib.Error:
+        remotes = []
+
+    for remote in remotes:
+        if remote.get_url() == url:
+            name = remote.get_name()
+
+    return name
+
 def _get_file_timestamp(gfile):
     try:
         info = gfile.query_info("time::modified", Gio.FileQueryInfoFlags.NONE, None)
@@ -242,6 +256,12 @@ def _is_ref_installed(fp_sys, ref):
     return False
 
 def _get_remote_sizes(fp_sys, remote, ref):
+    # Since 0.11.4, this info is part of FlatpakRemoteRef
+    # dl_s = ref.props.download_size
+    # inst_s = ref.props.installed_size
+    # But it appears to always contain identical dl and installed sizes,
+    # so avoid it for now.
+
     try:
         success, dl_s, inst_s = fp_sys.fetch_remote_size_sync(remote,
                                                               ref,
@@ -290,7 +310,7 @@ def _add_ref_to_task(fp_sys, task, ref, needs_update=False):
         dl_s, inst_s = _get_remote_sizes(fp_sys, ref.get_remote_name(), ref)
 
         task.download_size += dl_s
-        task.install_size = inst_s
+        task.install_size += inst_s
     elif task.type == "remove":
         _add_to_list(task.to_remove, ref)
         task.freed_size += _get_installed_size(fp_sys, ref)
@@ -307,12 +327,51 @@ def _add_ref_to_task(fp_sys, task, ref, needs_update=False):
         else:
             task.freed_size += current_inst_s - remote_inst_s
 
+def _find_remote_ref_from_list(fp_sys, remote_name, basic_ref, nofail=False):
+    remote_ref = None
+
+    all_refs = fp_sys.list_remote_refs_sync(remote_name, None)
+
+    ref_str = basic_ref.format_ref()
+
+    for ref in all_refs:
+        if ref_str == ref.format_ref():
+            remote_ref = ref
+            break
+
+    if remote_ref == None:
+        try:
+            remote_ref = fp_sys.fetch_remote_ref_sync(remote_name,
+                                                      basic_ref.get_kind(),
+                                                      basic_ref.get_name(),
+                                                      basic_ref.get_arch(),
+                                                      basic_ref.get_branch(),
+                                                      None)
+        except GLib.Error as e:
+            if nofail:
+                remote_ref = Flatpak.RemoteRef(remote_name=remote_name,
+                                               kind=basic_ref.get_kind(),
+                                               arch=basic_ref.get_arch(),
+                                               branch=basic_ref.get_branch(),
+                                               name=basic_ref.get_name(),
+                                               commit=basic_ref.get_commit(),
+                                               collection_id=basic_ref.get_collection_id())
+    return remote_ref
+
 def _get_runtime_ref(fp_sys, remote_name, ref):
     runtime_ref = None
     ref_string = None
 
     try:
-        meta = fp_sys.fetch_remote_metadata_sync(remote_name, ref, None)
+        # Since 0.11.4 a FlatpakRemoteRef contains its metadata, so this
+        # query is unnecessary.
+        try:
+            meta = ref.props.metadata
+
+            if meta == None:
+                raise AttributeError
+        except AttributeError:
+            meta = fp_sys.fetch_remote_metadata_sync(remote_name, ref, None)
 
         keyfile = GLib.KeyFile.new()
 
@@ -329,13 +388,11 @@ def _get_runtime_ref(fp_sys, remote_name, ref):
 
         # prefer the same-remote's runtimes
         try:
-            runtime_ref = fp_sys.fetch_remote_ref_sync(remote_name,
-                                                       basic_ref.get_kind(),
-                                                       basic_ref.get_name(),
-                                                       basic_ref.get_arch(),
-                                                       basic_ref.get_branch(),
-                                                       None)
-        except GLib.Error as e:
+            runtime_ref = _find_remote_ref_from_list(fp_sys, remote_name, basic_ref)
+
+            if runtime_ref:
+                print("Found runtime ref '%s' in remote %s" % (runtime_ref.format_ref(), remote_name))
+        except GLib.Error:
             pass
         # if nothing is found, check other remotes
         if runtime_ref == None:
@@ -348,16 +405,12 @@ def _get_runtime_ref(fp_sys, remote_name, ref):
                 print("Looking for %s in alternate remote %s" % (ref_string, other_remote_name))
 
                 try:
-                    runtime_ref = fp_sys.fetch_remote_ref_sync(other_remote_name,
-                                                               basic_ref.get_kind(),
-                                                               basic_ref.get_name(),
-                                                               basic_ref.get_arch(),
-                                                               basic_ref.get_branch(),
-                                                               None)
+                    runtime_ref = _find_remote_ref_from_list(fp_sys, other_remote_name, basic_ref)
                 except GLib.Error:
                     continue
 
                 if runtime_ref:
+                    print("Found runtime ref '%s' in remote %s" % (runtime_ref.format_ref(), other_remote_name))
                     break
             if runtime_ref == None:
                 raise Exception("Could not locate runtime '%s' in any registered remotes" % ref_string)
@@ -367,63 +420,27 @@ def _get_runtime_ref(fp_sys, remote_name, ref):
 
     return runtime_ref
 
-def _get_remote_related_refs(fp_sys, remote, ref):
+def _get_remote_related_refs(fp_sys, remote_name, ref):
     return_refs = []
 
     try:
-        related_refs = fp_sys.list_remote_related_refs_sync(remote,
+        related_refs = fp_sys.list_remote_related_refs_sync(remote_name,
                                                             ref.format_ref(),
                                                             None)
 
         for related_ref in related_refs:
-            real_ref = None
-
             if not related_ref.should_download():
                 continue
 
-            print("Looking for related ref %s in remote %s" % (related_ref.get_name(), remote))
+            print("Found related ref '%s' in remote %s" % (related_ref.format_ref(), remote_name))
 
-            try:
-                real_ref = fp_sys.fetch_remote_ref_sync(remote,
-                                                        related_ref.get_kind(),
-                                                        related_ref.get_name(),
-                                                        related_ref.get_arch(),
-                                                        related_ref.get_branch(),
-                                                        None)
-            except GLib.Error:
-                pass
+            # Convert to a RemoteRef so that later functions know what remote
+            # this is from.  Related refs should never fail from the given remote,
+            # but if they're no-enumerate (as .flatpakref installs can be) then a listing
+            # will be empty, so we have ..._from_list construct a synthetic one (nofail).
+            remote_ref = _find_remote_ref_from_list(fp_sys, remote_name, related_ref, nofail=True)
 
-            if real_ref == None:
-                for other_remote in fp_sys.list_remotes():
-                    other_remote_name = other_remote.get_name()
-
-                    if other_remote_name == remote:
-                        continue
-
-                    print("Looking for related ref %s in alternate remote %s" % (related_ref.get_name(), other_remote_name))
-
-                    try:
-                        real_ref = fp_sys.fetch_remote_ref_sync(other_remote_name,
-                                                                related_ref.get_kind(),
-                                                                related_ref.get_name(),
-                                                                related_ref.get_arch(),
-                                                                related_ref.get_branch(),
-                                                                None)
-                    except GLib.Error:
-                        continue
-
-                    if real_ref:
-                        break
-            if real_ref == None:
-                real_ref = Flatpak.RemoteRef(remote_name=remote,
-                                             kind=related_ref.get_kind(),
-                                             arch=related_ref.get_arch(),
-                                             branch=related_ref.get_branch(),
-                                             name=related_ref.get_name(),
-                                             commit=related_ref.get_commit(),
-                                             collection_id=related_ref.get_collection_id())
-
-            return_refs.append(real_ref)
+            return_refs.append(remote_ref)
     except GLib.Error as e:
         raise Exception("Could not determine remote related refs for app: %s" % e.message)
 
@@ -468,13 +485,10 @@ def _get_theme_refs(fp_sys, remote_name, ref):
             for matching_ref in matching_refs:
                 if matching_ref.get_arch() != ref.get_arch():
                     continue
-                try:
-                    if float(matching_ref.get_branch()) > float(ref.get_branch()):
-                        continue
-                except ValueError:
-                    continue
 
                 theme_ref = matching_ref
+                print("Found theme ref '%s' in remote %s" % (theme_ref.format_ref(), remote_name))
+                break
 
             # if nothing is found, check other remotes
             if theme_ref == None:
@@ -503,13 +517,10 @@ def _get_theme_refs(fp_sys, remote_name, ref):
                     for matching_ref in matching_refs:
                         if matching_ref.get_arch() != ref.get_arch():
                             continue
-                        try:
-                            if float(matching_ref.get_branch()) > float(ref.get_branch()):
-                                continue
-                        except ValueError:
-                            continue
 
                         theme_ref = matching_ref
+                        print("Found theme ref '%s' in alternate remote %s" % (theme_ref.format_ref(), other_remote_name))
+                        break
 
                     if theme_ref:
                         break
@@ -531,16 +542,6 @@ def _get_installed_related_refs(fp_sys, remote, ref):
         related_refs = fp_sys.list_installed_related_refs_sync(remote,
                                                                ref.format_ref(),
                                                                None)
-
-        for related_ref in related_refs:
-            real_ref = fp_sys.fetch_remote_ref_sync(remote,
-                                                    related_ref.get_kind(),
-                                                    related_ref.get_name(),
-                                                    related_ref.get_arch(),
-                                                    related_ref.get_branch(),
-                                                    None)
-
-            return_refs.append(real_ref)
     except GLib.Error as e:
         raise Exception("Could not determine installed refs for app: %s" % e.message)
 
@@ -602,16 +603,10 @@ def _pick_refs_for_installation(task):
     remote_name = pkginfo.remote
 
     try:
-        remote_ref = Flatpak.RemoteRef(remote_name=remote_name,
-                                       kind=ref.get_kind(),
-                                       arch=ref.get_arch(),
-                                       branch=ref.get_branch(),
-                                       name=ref.get_name(),
-                                       commit=ref.get_commit(),
-                                       collection_id=ref.get_collection_id())
+        remote_ref = _find_remote_ref_from_list(fp_sys, remote_name, ref, nofail=True)
+        print("Selected ref to install: '%s' in remote %s" % (remote_ref.format_ref(), remote_name))
 
         _add_ref_to_task(fp_sys, task, remote_ref)
-
         update_list = fp_sys.list_installed_refs_for_update(None)
 
         runtime_ref = _get_runtime_ref(fp_sys, remote_name, remote_ref)
@@ -721,6 +716,22 @@ def find_pkginfo(cache, string):
 
     return None
 
+def generate_uncached_pkginfos(cache):
+    fp_sys = get_fp_sys()
+
+    try:
+        for remote in fp_sys.list_remotes():
+            remote_name = remote.get_name()
+
+            for ref in fp_sys.list_installed_refs_by_kind(Flatpak.RefKind.APP, None):
+                # All remotes will see installed refs, but the installed refs will always
+                # report their correct origin, so only add installed refs when they match the remote.
+                if ref.get_origin() == remote_name:
+                    _add_package_to_cache(cache, ref, remote.get_url(), True)
+
+    except GLib.Error as e:
+        print("PearlInstall: flatpak - could not check for uncached pkginfos", e.message)
+
 def pkginfo_is_installed(pkginfo):
     fp_sys = get_fp_sys()
 
@@ -777,11 +788,33 @@ def _pkginfo_from_file_thread(cache, file, callback):
 
         try:
             ref = fp_sys.install_ref_file(gb, None)
+        except GLib.Error as e:
+            if e.code == Flatpak.Error.ALREADY_INSTALLED:
+                try:
+                    kf = GLib.KeyFile()
+                    if kf.load_from_file(path, GLib.KeyFileFlags.NONE):
+                        name = kf.get_string("Flatpak Ref", "Name")
+                        branch = kf.get_string("Flatpak Ref", "Branch")
+                        url = kf.get_string("Flatpak Ref", "Url")
+                        if name and branch and url:
+                            basic_ref = Flatpak.Ref.parse("app/%s/%s/%s" % (name, Flatpak.get_default_arch(), branch))
+                            remote_name = _get_remote_name_by_url(fp_sys, url)
 
-            if ref:
+                            ref = Flatpak.RemoteRef(remote_name=remote_name,
+                                                    kind=basic_ref.get_kind(),
+                                                    arch=basic_ref.get_arch(),
+                                                    branch=basic_ref.get_branch(),
+                                                    name=basic_ref.get_name())
+                except GLib.Error:
+                    print("PearlInstall: flatpak package already installed, but an error occurred finding it")
+            elif e.code != Gio.DBusError.ACCESS_DENIED: # user cancelling auth prompt for adding a remote
+                print("PearlInstall: could not read .flatpakref file: %s" % e.message)
+                dialogs.show_flatpak_error(e.message)
+
+        if ref:
+            try:
                 remote_name = ref.get_remote_name()
                 remote = fp_sys.get_remote_by_name(remote_name, None)
-                print("remote", remote.get_name())
 
                 # Add the ref's remote if it doesn't already exist
                 _process_remote(cache, fp_sys, remote, Flatpak.get_default_arch())
@@ -848,20 +881,8 @@ def _pkginfo_from_file_thread(cache, file, callback):
 
                                 fp_sys.drop_caches(None)
                         os.unlink(file.name)
-        except GLib.Error as e:
-            pkginfo = None
-
-            if e.code == Flatpak.Error.ALREADY_INSTALLED:
-                try:
-                    kf = GLib.KeyFile()
-                    if kf.load_from_file(path, GLib.KeyFileFlags.NONE):
-                        name = kf.get_string("Flatpak Ref", "Name")
-                        if name:
-                            pkginfo = find_pkginfo(cache, name)
-                except GLib.Error:
-                    print("PearlInstall: flatpak package already installed, but an error occurred finding it")
-            elif e.code != Gio.DBusError.ACCESS_DENIED: # user cancelling auth prompt for adding a remote
-                print("PearlInstall: could not read .flatpakref file: %s" % e.message)
+            except GLib.Error as e:
+                print("PearlInstall: could not process .flatpakref file: %s" % e.message)
                 dialogs.show_flatpak_error(e.message)
 
     GLib.idle_add(callback, pkginfo, priority=GLib.PRIORITY_DEFAULT)
@@ -1058,6 +1079,8 @@ class MetaTransaction():
         GLib.idle_add(self.task.error_cleanup_cb, self.task)
 
     def on_flatpak_finished(self):
+        self.fp_sys.drop_caches(None)
+
         GLib.idle_add(self.task.finished_cleanup_cb, self.task)
 
 
