@@ -148,6 +148,7 @@ class Installer:
             self.inited = True
 
             GObject.idle_add(self.initialize_appstream)
+            self.generate_uncached_pkginfos(self.cache)
 
             return True
 
@@ -185,6 +186,8 @@ class Installer:
 
         GObject.idle_add(self.initialize_appstream)
 
+        self.generate_uncached_pkginfos(self.cache)
+
         print('Full installer startup took %0.3f ms' % ((time.time() - self.startup_timer) * 1000.0))
 
         if self._init_cb:
@@ -316,8 +319,8 @@ class Installer:
 
     def list_flatpak_remotes(self):
         """
-        Returns a list of tuples of (remote name, remote title).  The remote_name
-        can be used to match with PkgInfo.remote and the title is for display.
+        Returns a list of FlatpakRemoteInfos.  The remote_name can be used to match
+        with PkgInfo.remote and the title is for display.
         """
         return _flatpak.list_remotes()
 
@@ -334,6 +337,15 @@ class Installer:
 
         return False
 
+    @print_timing
+    def generate_uncached_pkginfos(self, cache):
+        """
+        Flatpaks installed from .flatpakref files may not actually be in the saved
+        pkginfo cache, specifically, if they're added from no-enumerate-marked remotes.
+        This gets run at startup to collect and generate their info.
+        """
+        _flatpak.generate_uncached_pkginfos(cache)
+
     @print_timing
     def initialize_appstream(self):
         """
