@@ -11,31 +11,24 @@ import urllib.request, urllib.parse, urllib.error
 import random
 from datetime import datetime
 import subprocess
-import platform
 import functools
 import requests
 import configobj
 import re
-import math
 
 import gi
 gi.require_version('Gtk', '3.0')
 gi.require_version('AppStream', '1.0')
 gi.require_version('XApp', '1.0')
-from gi.repository import Gtk, Gdk, GdkPixbuf, GObject, GLib, Gio, XApp
+gi.require_version('Flatpak', '1.0')
+from gi.repository import Gtk, Gdk, GdkPixbuf, GObject, GLib, Gio, Flatpak, XApp
 
-from pearlcommon.installer import installer
-from pearlcommon.installer.misc import print_timing
+from installer import installer
+from misc import print_timing
 import reviews
 import housekeeping
 
-LIST_ICON_SIZE = 48
-FEATURED_ICON_SIZE = LIST_ICON_SIZE
-DETAILS_ICON_SIZE = 64
-SCREEN_THUMB_WIDTH = 100
-SCREENSHOT_WIDTH = 625
-
-FALLBACK_PACKAGE_ICON_PATH = "/usr/share/pearllinux/pearlinstall/data/available.png"
+ICON_SIZE = 48
 
 #Hardcoded mouse back button key for button-press-event
 #May not work on all mice
@@ -86,106 +79,6 @@ ALIASES['pia-manager'] = "PIA Manager"
 ALIASES['skypeforlinux'] = "Skype"
 ALIASES['google-earth-pro-stable'] = "Google Earth"
 ALIASES['whatsapp-desktop'] = "WhatsApp"
-ALIASES['wine-installer'] = "Wine"
-
-class AsyncImage(Gtk.Image):
-    def __init__(self, icon_string=None, width=DETAILS_ICON_SIZE, height=DETAILS_ICON_SIZE):
-        super(AsyncImage, self).__init__()
-
-        self.path = None
-        self.cancellable = None
-        self.loader = None
-
-        self.connect("destroy", self.on_destroyed)
-
-        if icon_string:
-            self.set_icon_string(icon_string, width, height)
-
-    def on_destroyed(self, widget, data=None):
-        if self.cancellable:
-            self.cancellable.cancel()
-
-    def set_icon_string(self, icon_string, width=DETAILS_ICON_SIZE, height=DETAILS_ICON_SIZE):
-        theme = Gtk.IconTheme.get_default()
-
-        self.original_width = width
-        self.original_height = height
-
-        # This keeps the icon's space occupied until loaded.
-        self.set_size_request(width, height)
-
-        if width != -1:
-            self.width = width * self.get_scale_factor()
-        else:
-            self.width = width
-
-        if height != -1:
-            self.height = height * self.get_scale_factor()
-        else:
-            self.height = height
-
-        self.cancellable = None
-        file = None
-
-        if os.path.isabs(icon_string):
-            self.path = icon_string
-            file = Gio.File.new_for_path(self.path)
-        elif icon_string.startswith("http"):
-            self.path = icon_string
-            file = Gio.File.new_for_uri(self.path)
-        elif theme.has_icon(icon_string):
-                info = theme.lookup_icon_for_scale(icon_string,
-                                                   self.height,
-                                                   self.get_scale_factor(),
-                                                   Gtk.IconLookupFlags.FORCE_SIZE)
-                if info:
-                    self.path = info.get_filename()
-                    file = Gio.File.new_for_path(self.path)
-
-        if file:
-            self.cancellable = Gio.Cancellable()
-            file.read_async(GLib.PRIORITY_DEFAULT, self.cancellable, self.on_read_finished)
-        else:
-            self.set_icon_string(FALLBACK_PACKAGE_ICON_PATH, self.original_width, self.original_height)
-
-    def on_read_finished(self, file, result, data=None):
-        if self.cancellable.is_cancelled():
-            return
-
-        stream = None
-
-        try:
-            stream = file.read_finish(result)
-        except GLib.Error as e:
-            print("AsyncIcon could not read icon file contents for loading (%s): %s" % (self.path, e.message))
-            self.set_icon_string(FALLBACK_PACKAGE_ICON_PATH, self.original_width, self.original_height)
-
-        if stream:
-            GdkPixbuf.Pixbuf.new_from_stream_at_scale_async(stream,
-                                                            self.width,
-                                                            self.height,
-                                                            True,
-                                                            self.cancellable,
-                                                            self.on_pixbuf_created)
-
-    def on_pixbuf_created(self, stream, result, data=None):
-        if self.cancellable.is_cancelled():
-            stream.close()
-            return
-
-        try:
-            pixbuf = GdkPixbuf.Pixbuf.new_from_stream_finish(result)
-
-            if pixbuf:
-                surface = Gdk.cairo_surface_create_from_pixbuf(pixbuf,
-                                                               self.get_scale_factor(),
-                                                               self.get_window())
-                self.set_from_surface(surface)
-        except GLib.Error as e:
-            print("AsyncIcon could not generate icon from file contents (%s): %s" % (self.path, e.message))
-            self.set_icon_string(FALLBACK_PACKAGE_ICON_PATH, self.original_width, self.original_height)
-
-        stream.close()
 
 class ScreenshotDownloader(threading.Thread):
     def __init__(self, application, pkginfo):
@@ -198,8 +91,8 @@ class ScreenshotDownloader(threading.Thread):
         self.application.screenshots = []
         # Add main screenshot
         try:
-            thumb = "https://community.linuxmint.com/thumbnail.php?w=250&pic=/var/www/community.linuxmint.com/img/screenshots/%s.png" % self.pkginfo.name
-            link = "https://community.linuxmint.com/img/screenshots/%s.png" % self.pkginfo.name
+            thumb = "https://community.pearllinux.com/thumbnail.php?w=250&pic=/var/www/community.pearllinux.com/img/screenshots/%s.png" % self.pkginfo.name
+            link = "https://community.pearllinux.com/img/screenshots/%s.png" % self.pkginfo.name
             if requests.head(link).status_code < 400:
                 num_screenshots += 1
 
@@ -526,7 +419,6 @@ class Application(Gtk.Application):
     PAGE_LIST = "list"
     PAGE_PACKAGE = "details"
     PAGE_LOADING = "loading"
-    PAGE_SEARCHING = "searching"
 
     def __init__(self):
         super(Application, self).__init__(application_id='com.pearllinux.pearlinstall',
@@ -534,10 +426,8 @@ class Application(Gtk.Application):
 
         self.gui_ready = False
 
-        self.low_res = self.get_low_res_screen()
-
         self.settings = Gio.Settings("com.pearllinux.install")
-        self.arch = platform.machine()
+        self.arch = Flatpak.get_default_arch()
 
         print("PearlInstall: Detected system architecture: '%s'" % self.arch)
 
@@ -659,14 +549,6 @@ class Application(Gtk.Application):
                                            _("An error occurred attempting to add the Flatpak repo."))
                 res = dialog.run()
                 dialog.destroy()
-            elif error == "no-flatpak-support":
-                dialog = Gtk.MessageDialog(self.main_window,
-                                           Gtk.DialogFlags.MODAL,
-                                           Gtk.MessageType.ERROR,
-                                           Gtk.ButtonsType.OK,
-                                           _("Flatpak support is not currently available. Try installing flatpak."))
-                res = dialog.run()
-                dialog.destroy()
             elif error == "cancel":
                 pass
 
@@ -676,41 +558,10 @@ class Application(Gtk.Application):
         self.add_categories()
         self.installer.init(self.on_installer_ready)
 
-    def on_pkginfo_from_uri_complete(self, pkginfo, error=None):
-        if error:
-            if error == "no-flatpak-support":
-                dialog = Gtk.MessageDialog(self.main_window,
-                                           Gtk.DialogFlags.MODAL,
-                                           Gtk.MessageType.ERROR,
-                                           Gtk.ButtonsType.OK,
-                                           _("Flatpak support is not currently available. Try installing flatpak and gir1.2-flatpak-1.0."))
-                res = dialog.run()
-                dialog.destroy()
-                self.finish_loading_visual()
-                return
+    def on_pkginfo_from_uri_complete(self, pkginfo):
         if pkginfo:
             self.show_package(pkginfo, self.PAGE_LANDING)
 
-    def get_low_res_screen(self):
-        display = Gdk.Display.get_default()
-        screen = display.get_default_screen()
-        pointer = display.get_default_seat().get_pointer()
-        if pointer:
-            position = pointer.get_position()
-            monitor_number = screen.get_monitor_at_point(position.x, position.y)
-        else:
-            monitor_number = 0
-        monitor = display.get_monitor(monitor_number)
-        height = monitor.get_geometry().height
-
-        # If it's less than our threshold than consider us 'low res'. The workarea being used is in
-        # app pixels, so hidpi will also be affected here regardless of device resolution.
-        if height < 768:
-            print("PearlInstall: low resolution detected on monitor %d (%dpx height), limiting window height." % (monitor_number, height))
-            return True
-
-        return False
-
     def create_window(self, starting_page):
         if self.main_window != None:
             print("PearlInstall: create_window called, but we already had one!")
@@ -730,25 +581,6 @@ class Application(Gtk.Application):
         self.main_window.connect("key-press-event", self.on_keypress)
         self.main_window.connect("button-press-event", self.on_buttonpress)
 
-        theme = Gtk.IconTheme.get_default()
-        for icon_name in ["application-x-deb", "file-roller"]:
-            if theme.has_icon(icon_name):
-                iconInfo = theme.lookup_icon_for_scale(icon_name,
-                                                       LIST_ICON_SIZE,
-                                                       self.main_window.get_scale_factor(),
-                                                       0)
-                if iconInfo and os.path.exists(iconInfo.get_filename()):
-                    global FALLBACK_PACKAGE_ICON_PATH
-                    FALLBACK_PACKAGE_ICON_PATH = iconInfo.get_filename()
-                    break
-
-        self.detail_view_icon = AsyncImage()
-        self.detail_view_icon.show()
-        self.builder.get_object("application_icon_holder").add(self.detail_view_icon)
-
-        self.main_screenshot = AsyncImage()
-        self.builder.get_object("box_main_screenshot").add(self.main_screenshot)
-
         self.status_label = self.builder.get_object("label_ongoing")
         self.progressbar = self.builder.get_object("progressbar1")
         self.progress_box = self.builder.get_object("progress_box")
@@ -758,9 +590,6 @@ class Application(Gtk.Application):
         self.active_tasks_spinner = self.builder.get_object("active_tasks_spinner")
         self.no_packages_found_label = self.builder.get_object("no_packages_found_label")
 
-        self.no_packages_found_refresh_button = self.builder.get_object("no_packages_found_refresh_button")
-        self.no_packages_found_refresh_button.connect("clicked", self.on_refresh_cache_clicked)
-
         self.progress_label = DottedProgressLabel()
         self.progress_box.pack_start(self.progress_label, False, False, 0)
         self.progress_label.show()
@@ -801,16 +630,6 @@ class Application(Gtk.Application):
         separator.show()
         submenu.append(separator)
 
-        self.refresh_cache_menuitem = Gtk.MenuItem(_("Refresh the list of packages"))
-        self.refresh_cache_menuitem.connect("activate", self.on_refresh_cache_clicked)
-        self.refresh_cache_menuitem.show()
-        self.refresh_cache_menuitem.set_sensitive(False)
-        submenu.append(self.refresh_cache_menuitem)
-
-        separator = Gtk.SeparatorMenuItem()
-        separator.show()
-        submenu.append(separator)
-
         about_menuitem = Gtk.MenuItem(_("About"))
         about_menuitem.connect("activate", self.open_about)
         about_menuitem.show()
@@ -857,6 +676,18 @@ class Application(Gtk.Application):
 
         self.app_list_stack = self.builder.get_object("app_list_stack")
 
+        self.generic_available_icon_path = "/usr/share/pearllinux/pearlinstall/data/available.png"
+        theme = Gtk.IconTheme.get_default()
+        for icon_name in ["application-x-deb", "file-roller"]:
+            if theme.has_icon(icon_name):
+                iconInfo = theme.lookup_icon(icon_name, ICON_SIZE, 0)
+                if iconInfo and os.path.exists(iconInfo.get_filename()):
+                    self.generic_available_icon_path = iconInfo.get_filename()
+                    break
+
+        self.generic_available_icon_pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(self.generic_available_icon_path,
+                                                                                    ICON_SIZE, ICON_SIZE)
+
         self.searchentry.grab_focus()
 
         self.listbox_categories = Gtk.ListBox()
@@ -865,57 +696,29 @@ class Application(Gtk.Application):
         self.listbox_categories_selected_id = self.listbox_categories.connect('row-activated',
                                                                               self.on_row_activated)
 
-    def refresh_cache(self):
-        self.builder.get_object("loading_spinner").start()
-        self.refresh_cache_menuitem.set_sensitive(False)
-
-        self.page_stack.set_visible_child_name(self.PAGE_LOADING)
-
-        self.installer.force_new_cache(self._on_refresh_cache_complete)
-
-    def _on_refresh_cache_complete(self):
-        self.add_categories()
-        self.installer.init(self.on_installer_ready)
-
-    def on_refresh_cache_clicked(self, widget, data=None):
-        self.refresh_cache()
-
     def on_installer_ready(self):
-        try:
-            self.process_matching_packages()
-            self.refresh_cache_menuitem.set_sensitive(True)
+        self.process_matching_packages()
 
-            self.apply_aliases()
+        self.apply_aliases()
 
-            self.load_featured_on_landing()
-            self.load_picks_on_landing()
-            self.load_categories_on_landing()
+        self.load_featured_on_landing()
+        self.load_picks_on_landing()
+        self.load_categories_on_landing()
 
-            self.sync_installed_apps()
-            self.update_conditional_widgets()
 
-            GObject.idle_add(self.finished_loading_packages)
+        self.sync_installed_apps()
+        self.update_conditional_widgets()
+        
+        GObject.idle_add(self.finished_loading_packages)
 
-            # Can take some time, don't block for it (these are categorizing packages based on apt info, not our listings)
-            GObject.idle_add(self.process_unmatched_packages)
+        # Can take some time, don't block for it (these are categorizing packages based on apt info, not our listings)
+        GObject.idle_add(self.process_unmatched_packages)
 
-            self.review_cache = reviews.ReviewCache()
-            housekeeping.run()
-        except GLib.Error as e:
-            print("Loading error: %s", e.message)
-            GObject.idle_add(self.refresh_cache)
+        self.review_cache = reviews.ReviewCache()
+        housekeeping.run()
 
     def load_featured_on_landing(self):
         box = self.builder.get_object("box_featured")
-
-        if self.low_res:
-            box.hide()
-
-            # This overrides the glade 800x600 defaults. 300 is excessively small so the window works
-            # out its own minimum height.
-            self.main_window.set_default_geometry(800, 300)
-            return
-
         for child in box.get_children():
             child.destroy()
 
@@ -966,9 +769,6 @@ class Application(Gtk.Application):
         box.show_all()
 
     def load_picks_on_landing(self):
-        if self.low_res:
-            self.builder.get_object("editor_picks_label").hide()
-
         box = self.builder.get_object("box_picks")
         for child in box.get_children():
             child.destroy()
@@ -999,7 +799,8 @@ class Application(Gtk.Application):
         random.shuffle(available)
         featured = 0
         for pkginfo in (available + installed):
-            icon = self.get_application_icon(pkginfo, FEATURED_ICON_SIZE)
+            icon = self.get_application_icon(pkginfo, ICON_SIZE)
+            icon = Gtk.Image.new_from_pixbuf(icon)
             tile = VerticalPackageTile(pkginfo, icon, self.installer)
             tile.connect("clicked", self.on_flowbox_item_clicked, pkginfo.pkg_hash)
             flowbox.insert(tile, -1)
@@ -1035,14 +836,12 @@ class Application(Gtk.Application):
         button.connect("clicked", self.category_button_clicked, self.picks_category)
         flowbox.insert(button, -1)
 
-        if self.installer.list_flatpak_remotes():
-            # Add flatpaks
-            button = Gtk.Button()
-            button.set_label(self.flatpak_category.name)
-            button.connect("clicked", self.category_button_clicked, self.flatpak_category)
-
-            flowbox.insert(button, -1)
+        # Add flatpaks
+        button = Gtk.Button()
+        button.set_label(self.flatpak_category.name)
+        button.connect("clicked", self.category_button_clicked, self.flatpak_category)
 
+        flowbox.insert(button, -1)
         box.pack_start(flowbox, True, True, 0)
         box.show_all()
 
@@ -1114,7 +913,6 @@ class Application(Gtk.Application):
                 self.show_package(self.current_pkginfo, self.previous_page)
             else:
                 del self.installer.cache[pkginfo.pkg_hash]
-                self.previous_page = self.PAGE_LANDING
                 self.go_back_action()
 
         for tile in (self.picks_tiles + self.category_tiles):
@@ -1151,12 +949,10 @@ class Application(Gtk.Application):
         return return_list
 
     def sync_installed_apps(self):
-        """ garbage collect any stale packages in this list (uninstalled somewhere else) """
+        # garbage collect any stale packages in this list (uninstalled somewhere else)
+        installed_packages = self.settings.get_strv(INSTALLED_APPS)
 
-        installed_packages = self.installer.cache.get_manually_installed_packages()
-        if not installed_packages:
-            installed_packages = self.settings.get_strv(INSTALLED_APPS)
-            installed_packages = self.modernize_installed_list(installed_packages)
+        installed_packages = self.modernize_installed_list(installed_packages)
 
         for pkg_hash in installed_packages:
             try:
@@ -1193,10 +989,11 @@ class Application(Gtk.Application):
             if (number == 1):
                 if os.path.exists(local_name):
                     try:
-                        self.main_screenshot.set_icon_string(local_name, SCREENSHOT_WIDTH, -1)
-                        self.main_screenshot.show()
+                        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(local_name, 625, -1)
+                        self.builder.get_object("main_screenshot").set_from_pixbuf(pixbuf)
+                        self.builder.get_object("main_screenshot").show()
                     except Exception:
-                        self.main_screenshot.hide()
+                        self.builder.get_object("main_screenshot").hide()
                         print("Invalid picture %s, deleting." % local_name)
                         os.unlink(local_name)
                         os.unlink(local_thumb)
@@ -1206,10 +1003,11 @@ class Application(Gtk.Application):
                         try:
                             name = os.path.join(SCREENSHOT_DIR, "%s_1.png" % pkg_name)
                             thumb = os.path.join(SCREENSHOT_DIR, "thumb_%s_1.png" % pkg_name)
+                            pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(thumb, 100, -1)
                             event_box = Gtk.EventBox()
-                            image = AsyncImage(thumb, SCREEN_THUMB_WIDTH, -1)
+                            image = Gtk.Image.new_from_pixbuf(pixbuf)
                             event_box.add(image)
-                            event_box.connect("button-release-event",
+                            event_box.connect("button-release-event", 
                                               self.on_screenshot_clicked,
                                               image,
                                               thumb,
@@ -1219,8 +1017,9 @@ class Application(Gtk.Application):
                         except Exception:
                             pass
                     try:
+                        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(local_thumb, 100, -1)
                         event_box = Gtk.EventBox()
-                        image = AsyncImage(local_thumb, SCREEN_THUMB_WIDTH, -1)
+                        image = Gtk.Image.new_from_pixbuf(pixbuf)
                         event_box.add(image)
                         event_box.connect("button-release-event",
                                           self.on_screenshot_clicked,
@@ -1236,7 +1035,8 @@ class Application(Gtk.Application):
 
     def on_screenshot_clicked(self, eventbox, event, image, local_thumb, local_name):
         # Set main screenshot
-        self.main_screenshot.set_icon_string(local_name, SCREENSHOT_WIDTH, -1)
+        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(local_name, 625, -1)
+        self.builder.get_object("main_screenshot").set_from_pixbuf(pixbuf)
 
     def category_button_clicked(self, button, category):
         self.show_category(category)
@@ -1256,7 +1056,7 @@ class Application(Gtk.Application):
         terms = searchentry.get_text()
 
         if terms != "":
-            self.show_search_results(terms)
+            self.show_search_results(terms);
 
     def on_entry_text_changed(self, entry):
         if self.search_changed_timer > 0:
@@ -1300,8 +1100,12 @@ class Application(Gtk.Application):
             dlg.set_license(gpl)
         except Exception as e:
             print(e)
+        try:
+            version = subprocess.check_output(["/usr/lib/pearllinux/common/version.py", "pearlinstall"]).decode()
+            dlg.set_version(version)
+        except Exception as e:
+            print(e)
 
-        dlg.set_version("__DEB_VERSION__")
         dlg.set_icon_name("pearlinstall")
         dlg.set_logo_icon_name("pearlinstall")
 
@@ -1325,16 +1129,14 @@ class Application(Gtk.Application):
 
         self.installer = installer.Installer()
 
-        from installer import cache
-
-        self.installer.cache = cache.PkgCache(self.installer.have_flatpak)
-        self.installer.force_new_cache()
-        self.installer.backend_table = {}
+        from installer.pkgCache import cache
 
+        self.installer.cache = cache.PkgCache()
+        self.installer.cache.force_new_cache()
         self.installer.initialize_appstream()
-        self.installer.generate_uncached_pkginfos(self.installer.cache)
 
         self.add_categories()
+        self.build_matched_packages()
         self.process_matching_packages()
         self.process_unmatched_packages()
 
@@ -1379,11 +1181,6 @@ class Application(Gtk.Application):
             if res == Gtk.ResponseType.NO:
                 return True
 
-        # kill -9 won't kill mp subprocesses, we have to do them ourselves.
-        housekeeping.kill()
-        if self.review_cache:
-            self.review_cache.kill()
-
         # Not happy with Python when it comes to closing threads, so here's a radical method to get what we want.
         os.system("kill -9 %s &" % os.getpid())
 
@@ -1742,7 +1539,7 @@ class Application(Gtk.Application):
         else:
             self.show_subcategories(category)
 
-        self.show_packages(category.pkginfos, from_search=False)
+        self.show_packages(category.pkginfos)
 
         self.update_conditional_widgets()
 
@@ -1788,18 +1585,40 @@ class Application(Gtk.Application):
     def on_row_activated(self, listbox, row):
         self.show_category(row.category)
 
-    def get_application_icon_string(self, pkginfo, size):
-        string = self.installer.get_icon(pkginfo, size)
-
-        if not string:
-            string = FALLBACK_PACKAGE_ICON_PATH
-
-        return string
-
     def get_application_icon(self, pkginfo, size):
-        icon_string = self.get_application_icon_string(pkginfo, size)
+        # Look in the icon theme first
+        theme = Gtk.IconTheme.get_default()
 
-        return AsyncImage(icon_string, size, size)
+        try:
+            for name in [pkginfo.name, pkginfo.name.split(":")[0], pkginfo.name.split("-")[0], pkginfo.name.split(".")[-1].lower()]:
+                if theme.has_icon(name):
+                    iconInfo = theme.lookup_icon(name, size, 0)
+                    if iconInfo and os.path.exists(iconInfo.get_filename()):
+                        return GdkPixbuf.Pixbuf.new_from_file_at_size(iconInfo.get_filename(), size, size)
+
+            # For Flatpak, look in Appstream and pearlinstall provided flatpak icons
+            if pkginfo.pkg_hash.startswith("f"):
+                icon_path = "/var/lib/flatpak/appstream/%s/%s/active/icons/64x64/%s.png" % (pkginfo.remote, pkginfo.arch, pkginfo.name)
+                if os.path.exists(icon_path):
+                    return GdkPixbuf.Pixbuf.new_from_file_at_size(icon_path, size, size)
+
+                icon_path = "/usr/share/pearllinux/pearlinstall/flatpak/icons/64x64/%s.png" % pkginfo.name
+                if os.path.exists(icon_path):
+                    return GdkPixbuf.Pixbuf.new_from_file_at_size(icon_path, size, size)
+
+            # Look in app-install-data and pixmaps
+            for extension in ['svg', 'png', 'xpm']:
+                icon_path = "/usr/share/app-install/icons/%s.%s" % (pkginfo.name, extension)
+                if os.path.exists(icon_path):
+                    return GdkPixbuf.Pixbuf.new_from_file_at_size(icon_path, size, size)
+
+                icon_path = "/usr/share/pixmaps/%s.%s" % (pkginfo.name, extension)
+                if os.path.exists(icon_path):
+                    return GdkPixbuf.Pixbuf.new_from_file_at_size(icon_path, size, size)
+        except GLib.Error:
+            pass
+
+        return self.generic_available_icon_pixbuf
 
     @print_timing
     def show_search_results(self, terms):
@@ -1824,9 +1643,7 @@ class Application(Gtk.Application):
         self.listbox_categories.hide()
         self.back_button.set_sensitive(True)
         self.previous_page = self.PAGE_LANDING
-        if self.page_stack.get_visible_child_name() != self.PAGE_SEARCHING:
-            self.builder.get_object("loading_spinner").start()
-            self.page_stack.set_visible_child_name(self.PAGE_SEARCHING)
+        self.page_stack.set_visible_child_name(self.PAGE_LIST)
 
         termsUpper = terms.upper()
         termsSplit = re.split(r'\W+', termsUpper)
@@ -1872,9 +1689,7 @@ class Application(Gtk.Application):
 
     def on_search_results_complete(self, results):
         self.clear_category_list()
-        self.page_stack.set_visible_child_name(self.PAGE_LIST)
-        self.builder.get_object("loading_spinner").stop()
-        self.show_packages(results, from_search=True)
+        self.show_packages(results)
 
     def on_flowbox_item_clicked(self, tile, data=None):
         # This ties the GtkButton.clicked signal for the Tile class
@@ -1915,7 +1730,7 @@ class Application(Gtk.Application):
         else:
             return (string)
 
-    def show_packages(self, pkginfos, from_search=False):
+    def show_packages(self, pkginfos):
         if self.one_package_idle_timer > 0:
             GObject.source_remove(self.one_package_idle_timer)
             self.one_package_idle_timer = 0
@@ -1926,17 +1741,10 @@ class Application(Gtk.Application):
         self.category_tiles = []
         if len(pkginfos) == 0:
             self.app_list_stack.set_visible_child_name("no-results")
-
             if self.current_category == self.active_tasks_category:
                 text = _("All operations complete")
-                self.no_packages_found_refresh_button.hide()
             else:
-                if from_search:
-                    text = _("No matching packages found")
-                    self.no_packages_found_refresh_button.hide()
-                else:
-                    text = _("No packages to show.\nThis may indicate a problem - try refreshing the cache.")
-                    self.no_packages_found_refresh_button.show()
+                text = _("No matching packages found")
 
             self.no_packages_found_label.set_markup("<big><b>%s</b></big>" % text)
         else:
@@ -2003,7 +1811,8 @@ class Application(Gtk.Application):
             self.one_package_idle_timer = 0
             return False
 
-        icon = self.get_application_icon(pkginfo, LIST_ICON_SIZE)
+        icon = self.get_application_icon(pkginfo, ICON_SIZE)
+        icon = Gtk.Image.new_from_pixbuf(icon)
 
         summary = self.installer.get_summary(pkginfo)
         summary = summary.replace("<", "&lt;")
@@ -2078,8 +1887,7 @@ class Application(Gtk.Application):
 
         self.builder.get_object("label_details").set_markup(details_markup)
 
-        icon_string = self.get_application_icon_string(pkginfo, DETAILS_ICON_SIZE)
-        self.detail_view_icon.set_icon_string(icon_string)
+        self.builder.get_object("application_icon").set_from_pixbuf(self.get_application_icon(pkginfo, 64))
         self.builder.get_object("application_name").set_label(self.installer.get_display_name(pkginfo))
         self.builder.get_object("application_summary").set_label(self.installer.get_summary(pkginfo))
         self.builder.get_object("application_package").set_label(pkginfo.name)
@@ -2149,7 +1957,7 @@ class Application(Gtk.Application):
             label_reviews.set_text(_("No reviews available"))
             frame_reviews.hide()
 
-        community_link = "https://community.linuxmint.com/software/view/%s" % pkginfo.name
+        community_link = "https://community.pearllinux.com/software/view/%s" % pkginfo.name
 
         self.builder.get_object("label_community") \
             .set_markup(_("Click <a href='%s'>here</a> to add your own review.") % community_link)
@@ -2163,15 +1971,16 @@ class Application(Gtk.Application):
         main_thumb = os.path.join(SCREENSHOT_DIR, "thumb_%s_1.png" % pkginfo.name)
         if os.path.exists(main_screenshot) and os.path.exists(main_thumb):
             try:
-                self.main_screenshot.set_icon_string(main_screenshot, SCREENSHOT_WIDTH, -1)
-                self.main_screenshot.show()
+                main_screenshot = GdkPixbuf.Pixbuf.new_from_file_at_size(main_screenshot, 625, -1)
+                self.builder.get_object("main_screenshot").set_from_pixbuf(main_screenshot)
+                self.builder.get_object("main_screenshot").show()
             except:
-                self.main_screenshot.hide()
+                self.builder.get_object("main_screenshot").hide()
                 os.unlink(main_screenshot)
             for i in range(2, 5):
                 self.add_screenshot(pkginfo.name, i)
         else:
-            self.main_screenshot.hide()
+            self.builder.get_object("main_screenshot").hide()
             downloadScreenshots = ScreenshotDownloader(self, pkginfo)
             downloadScreenshots.start()
 
@@ -2205,8 +2014,6 @@ class Application(Gtk.Application):
         # Load package info
         style_context = self.action_button.get_style_context()
 
-        action_button_description = ""
-
         if task.info_ready_status == task.STATUS_OK:
             if task.type == "remove":
                 action_button_label = _("Remove")
