@@ -1,6 +1,6 @@
 import os
 import threading
-import json
+import pickle
 import requests
 import multiprocessing
 
@@ -10,29 +10,16 @@ from gi.repository import GLib
 
 from misc import print_timing
 
-REVIEWS_CACHE = os.path.join(GLib.get_user_cache_dir(), "pearlinstall", "reviews.json")
-
-class Review(object):
-    def __init__(self, packagename, date, username, rating, comment):
-        self.date = date
-        self.packagename = packagename
-        self.username = username
-        self.rating = int(rating)
-        self.comment = comment
-
-    @classmethod
-    def from_json(cls, json_data:dict):
-        return cls(**json_data)
-
+REVIEWS_CACHE = os.path.join(GLib.get_user_cache_dir(), "pearlinstall", "reviews.cache")
 
 class ReviewInfo:
-    def __init__(self, name, score=0, avg_rating=0, num_reviews=0):
+    def __init__(self, name):
         self.name = name
         self.reviews = []
         self.categories = []
-        self.score = score
-        self.avg_rating = avg_rating
-        self.num_reviews = num_reviews
+        self.score = 0
+        self.avg_rating = 0
+        self.num_reviews = 0
 
     def update_stats(self):
         points = 0
@@ -46,33 +33,24 @@ class ReviewInfo:
             self.avg_rating = round(float(sum_rating) / float(self.num_reviews), 1)
         self.score = points
 
-    @classmethod
-    def from_json(cls, json_data:dict):
-        reviews = list(map(Review.from_json, json_data["reviews"]))
-        inst = cls(json_data["name"],
-                   json_data["score"],
-                   json_data["avg_rating"],
-                   json_data["num_reviews"])
-        inst.reviews = reviews
+class Review(object):
+    __slots__ = 'date', 'packagename', 'username', 'rating', 'comment', 'package' #To remove __dict__ memory overhead
 
-        return inst
+    def __init__(self, packagename, date, username, rating, comment):
+        self.date = date
+        self.packagename = packagename
+        self.username = username
+        self.rating = int(rating)
+        self.comment = comment
+        self.package = None
 
-class JsonObject(object):
+class PickleObject(object):
     def __init__(self, cache, size):
-        super(JsonObject, self).__init__()
+        super(PickleObject, self).__init__()
 
         self.cache = cache
         self.size = int(size)
 
-    @classmethod
-    def from_json(cls, json_data: dict):
-        new_dict = {}
-        for key in json_data["cache"].keys():
-            info_data = json_data["cache"][key]
-            new_dict[key] = ReviewInfo.from_json(info_data)
-
-        return cls(new_dict, json_data["size"])
-
 class ReviewCache(object):
     @print_timing
     def __init__(self):
@@ -82,17 +60,8 @@ class ReviewCache(object):
 
         self._reviews, self._size = self._load_cache()
 
-        self.proc = None
-
         self._update_cache()
 
-    def kill(self):
-        try:
-            self.proc.terminate()
-            self.proc = None
-        except AttributeError as e:
-            pass
-
     def keys(self):
         with self._cache_lock:
             return self._reviews.keys()
@@ -130,10 +99,11 @@ class ReviewCache(object):
         finally:
             if path != None:
                 try:
-                    with path.open(mode='r', encoding="utf8") as f:
-                        json_object = JsonObject.from_json(json.load(f))
-                        cache = json_object.cache
-                        size = json_object.size
+                    with path.open(mode='rb') as f:
+                        pobj = pickle.load(f)
+
+                        cache = pobj.cache
+                        size = pobj.size
                 except Exception as e:
                     print("PearlInstall: Cannot open reviews cache: %s" % str(e))
                     cache = {}
@@ -151,11 +121,11 @@ class ReviewCache(object):
             path = None
         finally:
             try:
-                with path.open(mode='w', encoding="utf8") as f:
-                    pobj = JsonObject(cache, size)
-                    json.dump(pobj, f, default=lambda o: o.__dict__, indent=4)
+                with path.open(mode='wb') as f:
+                    pobj = PickleObject(cache, size)
+                    pickle.dump(pobj, f)
             except Exception as e:
-                print("PearlInstall: Could not save review cache: %s" % str(e))
+                print("PearlInstall: Could not save review cache:" % str(e))
 
     def _update_cache(self):
         thread = threading.Thread(target=self._update_reviews_thread)
@@ -170,12 +140,10 @@ class ReviewCache(object):
         success = multiprocessing.Value('b', False)
 
         current_size = multiprocessing.Value('d', self._size)
-        self.proc = multiprocessing.Process(target=self._update_cache_process, args=(success, current_size))
-        self.proc.start()
+        proc = multiprocessing.Process(target=self._update_cache_process, args=(success, current_size))
 
-        self.proc.join()
-
-        self.proc = None
+        proc.start()
+        proc.join()
 
         if success.value == True:
             with self._cache_lock:
@@ -185,12 +153,12 @@ class ReviewCache(object):
         new_reviews = {}
 
         try:
-            r = requests.head("https://community.linuxmint.com/data/new-reviews.list")
+            r = requests.head("https://community.pearllinux.com/data/new-reviews.list")
 
             if r.status_code == 200:
                 if int(r.headers.get("content-length")) != current_size.value:
 
-                    r = requests.get("https://community.linuxmint.com/data/new-reviews.list")
+                    r = requests.get("https://community.pearllinux.com/data/new-reviews.list")
 
                     last_package = None
 
@@ -203,6 +171,7 @@ class ReviewCache(object):
                             if last_package != None and last_package.name == elements[0]:
                                 #Comment is on the same package as previous comment.. no need to search for the package
                                 last_package.reviews.append(review)
+                                review.package = last_package
                             else:
                                 if last_package is not None:
                                     last_package.update_stats()
@@ -215,6 +184,7 @@ class ReviewCache(object):
 
                                 last_package = package
                                 package.reviews.append(review)
+                                review.package = package
 
                     if last_package is not None:
                         last_package.update_stats()
